@@ -47,10 +47,28 @@ static int PendingHash_Match(const void *elem, const void *key)
 	return (elem == key);
 }
 
+/*
+ * Determine if a value type corresponds to a C type that must be output.
+ */
+static int IsCValueType(ILClass *classInfo)
+{
+	if((ILClass_Attrs(classInfo) & IL_META_TYPEDEF_TYPE_BITS) != 0)
+	{
+		return 1;
+	}
+	else if(!strncmp(ILClass_Name(classInfo), "array ", 6))
+	{
+		return 1;
+	}
+	else
+	{
+		return 0;
+	}
+}
+
 void CTypeMarkForOutput(ILGenInfo *info, ILType *type)
 {
 	ILClass *classInfo;
-	const char *name;
 	unsigned long param;
 	unsigned long numParams;
 
@@ -58,11 +76,7 @@ void CTypeMarkForOutput(ILGenInfo *info, ILType *type)
 	if(ILType_IsValueType(type))
 	{
 		classInfo = ILType_ToValueType(type);
-		name = ILClass_Name(classInfo);
-		if(!strncmp(name, "struct ", 7) ||
-		   !strncmp(name, "union ", 6) ||
-		   !strncmp(name, "enum ", 5) ||
-		   !strncmp(name, "array ", 6))
+		if(IsCValueType(classInfo))
 		{
 			/* Scan up to find the outermost nesting level */
 			while(ILClass_NestedParent(classInfo) != 0)
@@ -154,6 +168,18 @@ void CGenOutputAttributes(ILGenInfo *info, FILE *stream, ILProgramItem *item)
 	}
 }
 
+/*
+ * Store to a global variable.
+ */
+static void StoreGlobal(ILGenInfo *info, ILClass *classInfo,
+						const char *name1, const char *name2,
+						FILE *stream)
+{
+	fputs("\tstsfld\tunsigned int32 ", stream);
+	ILDumpClassName(stream, info->image, classInfo, IL_DUMP_QUOTE_NAMES);
+	fprintf(stream, "::'%s.%s'\n", name1, name2);
+}
+
 /*
  * Output the definition of a pending class and mark any other
  * classes that it may depend upon.
@@ -166,6 +192,10 @@ static void OutputPendingClass(ILGenInfo *info, ILClass *classInfo,
 	ILField *field;
 	ILClassLayout *classLayout;
 	ILFieldLayout *fieldLayout;
+	ILType *type;
+	CTypeLayoutInfo layout;
+	ILNode *node;
+	int first;
 
 	/* Ignore class references, which will normally be struct's or
 	   union's that weren't fully defined in the current module */
@@ -174,10 +204,19 @@ static void OutputPendingClass(ILGenInfo *info, ILClass *classInfo,
 		return;
 	}
 
+	/* Determine if the type is complex */
+	type = ILClassToType(classInfo);
+	CTypeGetLayoutInfo(type, &layout);
+
 	/* Output the class header.  We assume that there are no interfaces
 	   because C structs, unions, etc do not need interfaces */
 	fputs(".class ", stream);
-	ILDumpFlags(stream, ILClass_Attrs(classInfo), ILTypeDefinitionFlags, 0);
+	ILDumpFlags(stream, ILClass_Attrs(classInfo) &
+						~IL_META_TYPEDEF_TYPE_BITS, ILTypeDefinitionFlags, 0);
+	if(layout.category == C_TYPECAT_COMPLEX)
+	{
+		fputs("beforefieldinit ", stream);
+	}
 	ILDumpIdentifier(stream, ILClass_Name(classInfo), 0, IL_DUMP_QUOTE_NAMES);
 	parent = ILClass_Parent(classInfo);
 	if(parent)
@@ -236,6 +275,71 @@ static void OutputPendingClass(ILGenInfo *info, ILClass *classInfo,
 		CTypeMarkForOutput(info, ILMember_Signature(field));
 	}
 
+	/* If the type is complex, then we need to output some static fields */
+	if(layout.category == C_TYPECAT_COMPLEX)
+	{
+		/* Output the "size.of" field, which defines the type's size */
+		fputs(".field public static initonly unsigned int32 'size.of'\n",
+			  stream);
+		if(CTypeIsStruct(type))
+		{
+			field = 0;
+			first = 1;
+			while((field = CTypeNextField(type, field)) != 0)
+			{
+				if(!first)
+				{
+					fprintf(stream,
+							".field public static initonly unsigned int32 "
+							"'%s.offset'\n", ILField_Name(field));
+				}
+				else
+				{
+					/* We don't need the offset of the first field,
+					   because it will always be zero */
+					first = 0;
+				}
+			}
+		}
+
+		/* Output the header for the static constructor */
+		fputs(".method private static hidebysig specialname rtspecialname "
+			  "void .cctor() cil managed\n{\n", stream);
+		info->stackHeight = 0;
+		info->maxStackHeight = 0;
+
+		/* Generate code to compute the size and field offsets */
+		node = CTypeCreateSizeNode(type);
+		ILNode_GenValue(node, info);
+		StoreGlobal(info, classInfo, "size", "of", stream);
+		ILGenAdjust(info, -1);
+		if(CTypeIsStruct(type))
+		{
+			field = 0;
+			first = 1;
+			while((field = CTypeNextField(type, field)) != 0)
+			{
+				if(!first)
+				{
+					node = CTypeCreateOffsetNode(type, ILField_Name(field));
+					ILNode_GenValue(node, info);
+					StoreGlobal(info, classInfo, ILField_Name(field),
+								"offset", stream);
+					ILGenAdjust(info, -1);
+				}
+				else
+				{
+					/* We don't need the offset of the first field,
+					   because it will always be zero */
+					first = 0;
+				}
+			}
+		}
+
+		/* Output the footer for the static constructor */
+		fprintf(stream, "\tret\n\t.maxstack %ld\n}\n", info->maxStackHeight);
+	}
+
 	/* Output the class footer */
 	fputs("} // class ", stream);
 	fputs(ILClass_Name(classInfo), stream);
