@@ -123,6 +123,12 @@ extern ILGenInfo CCCodeGen;
 extern ILNode *CCParseTree;
 extern ILNode *CCParseTreeEnd;
 
+/*
+ * Standalone attribute blocks that aren't attached to a class.
+ */
+extern ILNode *CCStandaloneAttrs;
+extern ILNode *CCStandaloneAttrsEnd;
+
 /*
  * Global definition scope.
  */
@@ -157,6 +163,11 @@ void CCPluginParseError(char *msg, char *text);
  */
 void CCPluginAddTopLevel(ILNode *node);
 
+/*
+ * Add a node to the standalone attribute list.
+ */
+void CCPluginAddStandaloneAttrs(ILNode *node);
+
 /*
  * Load a library into the compiler.
  */
