@@ -65,6 +65,7 @@ int const CCPluginForceStdlib = 0;
 
 int CCPluginInit(void)
 {
+	CSMetadataOnly = 1;
 	/* Nothing to do here */
 	return 1;
 }
@@ -88,10 +89,13 @@ void CCPluginRestart(FILE *infile)
 
 void CCPluginSemAnalysis(void)
 {
-	/* Perform type gathering only */
+	/* Perform type gathering */
 	CCCodeGen.typeGather = 1;
 	CCParseTree = CSTypeGather(&CCCodeGen, CCGlobalScope, CCParseTree);
 	CCCodeGen.typeGather = 0;
+
+	/* Perform semantic analysis */
+	ILNode_SemAnalysis(CCParseTree, &CCCodeGen, &CCParseTree);
 }
 
 void CCPluginPostCodeGen(void)
@@ -139,6 +143,84 @@ static void Indent(FILE *stream, int indent)
 	}
 }
 
+/*
+ * Dump a string, with XML quoting.
+ */
+static void DumpString(const char *str, FILE *stream)
+{
+	int ch;
+	if(!str)
+	{
+		return;
+	}
+	while((ch = *str++) != '\0')
+	{
+		if(ch == '<')
+		{
+			fputs("&lt;", stream);
+		}
+		else if(ch == '>')
+		{
+			fputs("&gt;", stream);
+		}
+		else if(ch == '&')
+		{
+			fputs("&amp;", stream);
+		}
+		else if(ch == '"')
+		{
+			fputs("&quot;", stream);
+		}
+		else if(ch == '\'')
+		{
+			fputs("&apos;", stream);
+		}
+		else
+		{
+			putc(ch, stream);
+		}
+	}
+}
+
+/*
+ * Dump a length-delimited string, with XML quoting.
+ */
+static void DumpStringN(const char *str, int len, FILE *stream)
+{
+	int ch;
+	if(!str)
+	{
+		return;
+	}
+	while(len-- > 0 && (ch = *str++) != '\0')
+	{
+		if(ch == '<')
+		{
+			fputs("&lt;", stream);
+		}
+		else if(ch == '>')
+		{
+			fputs("&gt;", stream);
+		}
+		else if(ch == '&')
+		{
+			fputs("&amp;", stream);
+		}
+		else if(ch == '"')
+		{
+			fputs("&quot;", stream);
+		}
+		else if(ch == '\'')
+		{
+			fputs("&apos;", stream);
+		}
+		else
+		{
+			putc(ch, stream);
+		}
+	}
+}
+
 /*
  * Dump a class name to a stream.
  */
@@ -148,10 +230,10 @@ static void DumpClassName(FILE *stream, ILClass *classInfo)
 	const char *namespace = ILClass_Namespace(classInfo);
 	if(namespace)
 	{
-		fputs(namespace, stream);
+		DumpString(namespace, stream);
 		putc('.', stream);
 	}
-	fputs(name, stream);
+	DumpString(name, stream);
 }
 
 /*
@@ -162,16 +244,16 @@ static void DumpClassNameOther(FILE *stream, ILClass *classInfo, ILClass *other)
 {
 	const char *name = ILClass_Name(classInfo);
 	const char *namespace = ILClass_Namespace(classInfo);
-	const char *namespace2 = ILClass_Namespace(other);
+	const char *namespace2 = (other ? ILClass_Namespace(other) : 0);
 	if(namespace)
 	{
 		if(!namespace2 || strcmp(namespace, namespace2) != 0)
 		{
-			fputs(namespace, stream);
+			DumpString(namespace, stream);
 			putc('.', stream);
 		}
 	}
-	fputs(name, stream);
+	DumpString(name, stream);
 }
 
 /*
@@ -198,7 +280,7 @@ static void DumpClassNameSP(FILE *stream, ILClass *classInfo)
 		}
 		putc('_', stream);
 	}
-	fputs(name, stream);
+	DumpString(name, stream);
 }
 
 /*
@@ -260,18 +342,89 @@ static void DumpDocComments(FILE *stream, ILNode *attrs, int indent)
 	}
 }
 
+/*
+ * Dump a specific attribute.
+ */
+static void DumpAttribute(FILE *stream, ILAttribute *attr,
+						  ILClass *owner, int indent)
+{
+	ILMethod *ctor;
+	ILType *signature;
+
+	/* Convert the attribute's type into a constructor reference */
+	ctor = ILProgramItemToMethod(ILAttributeTypeAsItem(attr));
+	if(!ctor)
+	{
+		return;
+	}
+	signature = ILMethod_Signature(ctor);
+
+	/* Output the attribute header */
+	Indent(stream, indent);
+	fputs("<Attribute>\n", stream);
+
+	/* Output the attribute name start */
+	Indent(stream, indent + 2);
+	fputs("<AttributeName>", stream);
+	DumpClassNameOther(stream, ILMethod_Owner(ctor), owner);
+
+	/* Output the parameters */
+	if(ILTypeNumParams(signature) != 0)
+	{
+		/* TODO: output the constant values for the parameters */
+		putc('(', stream);
+		putc('?', stream);
+		putc(')', stream);
+	}
+
+	/* Output the attribute name end */
+	fputs("</AttributeName>\n", stream);
+
+	/* Output the attribute footer */
+	Indent(stream, indent + 2);
+	fputs("<Excluded>0</Excluded>\n", stream);
+	Indent(stream, indent);
+	fputs("</Attribute>\n", stream);
+}
+
 /*
  * Dump the attributes for an item.
  */
-static void DumpAttributes(FILE *stream, ILNode *attrs, int indent)
+static void DumpAttributes(FILE *stream, ILProgramItem *item, int indent)
 {
-	if(attrs && yyisa(attrs, ILNode_AttributeTree))
+	ILAttribute *attr = ILProgramItemNextAttribute(item, 0);
+	ILClass *owner;
+	ILMember *member;
+	if(attr)
 	{
-		attrs = ((ILNode_AttributeTree *)attrs)->sections;
+		owner = ILProgramItemToClass(item);
+		if(!owner)
+		{
+			member = ILProgramItemToMember(item);
+			if(member)
+			{
+				owner = ILMember_Owner(member);
+			}
+			else
+			{
+				owner = 0;
+			}
+		}
+		Indent(stream, indent);
+		fputs("<Attributes>\n", stream);
+		do
+		{
+			DumpAttribute(stream, attr, owner, indent + 2);
+		}
+		while((attr = ILProgramItemNextAttribute(item, attr)) != 0);
+		Indent(stream, indent);
+		fputs("</Attributes>\n", stream);
+	}
+	else
+	{
+		Indent(stream, indent);
+		fputs("<Attributes/>\n", stream);
 	}
-	/* TODO */
-	Indent(stream, indent);
-	fputs("<Attributes/>\n", stream);
 }
 
 /*
@@ -391,7 +544,7 @@ static void GenerateDocsForField(FILE *stream, ILNode_FieldDeclaration *decl,
 		/* Output the member header */
 		Indent(stream, indent);
 		fputs("<Member MemberName=\"", stream);
-		fputs(ILField_Name(field), stream);
+		DumpString(ILField_Name(field), stream);
 		fputs("\">\n", stream);
 
 		/* Output the signature in ILASM form */
@@ -399,18 +552,18 @@ static void GenerateDocsForField(FILE *stream, ILNode_FieldDeclaration *decl,
 		fputs("<MemberSignature Language=\"ILASM\" Value=\".field ", stream);
 		ILDumpFlags(stream, ILField_Attrs(field), ILFieldDefinitionFlags, 0);
 		ILDumpType(stream, ILProgramItem_Image(field),
-				   ILField_Type(field), 0);
+				   ILField_Type(field), IL_DUMP_XML_QUOTING);
 		putc(' ', stream);
-		fputs(ILField_Name(field), stream);
+		DumpString(ILField_Name(field), stream);
 		fputs("\"/>\n", stream);
 
 		/* Output the signature in C# form */
 		Indent(stream, indent + 2);
 		fputs("<MemberSignature Language=\"C#\" Value=\"", stream);
 		ILDumpFlags(stream, decl->modifiers, CSharpFieldFlags, 0);
-		fputs(CSTypeToName(ILField_Type(field)), stream);
+		DumpString(CSTypeToName(ILField_Type(field)), stream);
 		putc(' ', stream);
-		fputs(ILField_Name(field), stream);
+		DumpString(ILField_Name(field), stream);
 		if(ILField_IsLiteral(field))
 		{
 			/* TODO: dump the constant value */
@@ -422,14 +575,15 @@ static void GenerateDocsForField(FILE *stream, ILNode_FieldDeclaration *decl,
 		fputs("<MemberType>Field</MemberType>\n", stream);
 
 		/* Dump the attributes for the field */
-		DumpAttributes(stream, decl->attributes, indent + 2);
+		DumpAttributes(stream, ILToProgramItem(field), indent + 2);
 
 		/* Output the field's type */
 		Indent(stream, indent + 2);
 		fputs("<ReturnValue>\n", stream);
 		Indent(stream, indent + 4);
-		fprintf(stream, "<ReturnType>%s</ReturnType>\n",
-				CSTypeToName(ILField_Type(field)));
+		fputs("<ReturnType>", stream);
+		DumpString(CSTypeToName(ILField_Type(field)), stream);
+		fputs("</ReturnType>\n", stream);
 		Indent(stream, indent + 2);
 		fputs("</ReturnValue>\n", stream);
 
@@ -470,7 +624,7 @@ static void GenerateDocsForEnum(FILE *stream,
 	/* Output the member header */
 	Indent(stream, indent);
 	fputs("<Member MemberName=\"", stream);
-	fputs(ILField_Name(field), stream);
+	DumpString(ILField_Name(field), stream);
 	fputs("\">\n", stream);
 
 	/* Output the signature in ILASM form */
@@ -478,18 +632,18 @@ static void GenerateDocsForEnum(FILE *stream,
 	fputs("<MemberSignature Language=\"ILASM\" Value=\".field ", stream);
 	ILDumpFlags(stream, ILField_Attrs(field), ILFieldDefinitionFlags, 0);
 	ILDumpType(stream, ILProgramItem_Image(field),
-			   ILField_Type(field), 0);
+			   ILField_Type(field), IL_DUMP_XML_QUOTING);
 	putc(' ', stream);
-	fputs(ILField_Name(field), stream);
+	DumpString(ILField_Name(field), stream);
 	fputs("\"/>\n", stream);
 
 	/* Output the signature in C# form */
 	Indent(stream, indent + 2);
 	fputs("<MemberSignature Language=\"C#\" Value=\"", stream);
-	fputs(ILField_Name(field), stream);
+	DumpString(ILField_Name(field), stream);
 	if(ILField_IsLiteral(field))
 	{
-		/* TODO: dump the constant value */
+		/* TODO */
 	}
 	fputs("\"/>\n", stream);
 
@@ -498,14 +652,15 @@ static void GenerateDocsForEnum(FILE *stream,
 	fputs("<MemberType>Field</MemberType>\n", stream);
 
 	/* Dump the attributes for the enumerated member */
-	DumpAttributes(stream, decl->attributes, indent + 2);
+	DumpAttributes(stream, ILToProgramItem(field), indent + 2);
 
 	/* Output the field's type */
 	Indent(stream, indent + 2);
 	fputs("<ReturnValue>\n", stream);
 	Indent(stream, indent + 4);
-	fprintf(stream, "<ReturnType>%s</ReturnType>\n",
-			CSTypeToName(ILField_Type(field)));
+	fputs("<ReturnType>", stream);
+	DumpString(CSTypeToName(ILField_Type(field)), stream);
+	fputs("</ReturnType>\n", stream);
 	Indent(stream, indent + 2);
 	fputs("</ReturnValue>\n", stream);
 
@@ -684,7 +839,7 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	/* Output the member header */
 	Indent(stream, indent);
 	fputs("<Member MemberName=\"", stream);
-	fputs(ILMethod_Name(method), stream);
+	DumpString(ILMethod_Name(method), stream);
 	fputs("\">\n", stream);
 
 	/* Is this method a constructor? */
@@ -701,7 +856,7 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	ILDumpFlags(stream, ILMethod_Attrs(method), ILMethodDefinitionFlags, 0);
 	signature = ILMethod_Signature(method);
 	ILDumpMethodType(stream, ILProgramItem_Image(method), signature,
-			         0, 0, ILMethod_Name(method), method);
+			         IL_DUMP_XML_QUOTING, 0, ILMethod_Name(method), method);
 	putc(' ', stream);
 	ILDumpFlags(stream, ILMethod_ImplAttrs(method),
 				ILMethodImplementationFlags, 0);
@@ -713,7 +868,7 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	ILDumpFlags(stream, decl->modifiers, CSharpMethodFlags, 0);
 	if(!isConstructor)
 	{
-		fputs(CSTypeToName(ILTypeGetReturn(signature)), stream);
+		DumpString(CSTypeToName(ILTypeGetReturn(signature)), stream);
 		putc(' ', stream);
 	}
 	if(isConstructor)
@@ -724,7 +879,7 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	{
 		methodName = ConvertOperatorNames(method);
 	}
-	fputs(methodName, stream);
+	DumpString(methodName, stream);
 	putc('(', stream);
 	num = ILTypeNumParams(signature);
 	for(param = 1; param <= num; ++param)
@@ -733,12 +888,12 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 		{
 			fputs(", ", stream);
 		}
-		fputs(CSTypeToName(ILTypeGetParam(signature, param)), stream);
+		DumpString(CSTypeToName(ILTypeGetParam(signature, param)), stream);
 		paramInfo = FindMethodParam(method, param);
 		if(paramInfo)
 		{
 			putc(' ', stream);
-			fputs(ILParameter_Name(paramInfo), stream);
+			DumpString(ILParameter_Name(paramInfo), stream);
 		}
 	}
 	putc(')', stream);
@@ -756,7 +911,7 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	}
 
 	/* Dump the attributes for the method */
-	DumpAttributes(stream, decl->attributes, indent + 2);
+	DumpAttributes(stream, ILToProgramItem(method), indent + 2);
 
 	/* Output the method's type */
 	Indent(stream, indent + 2);
@@ -765,8 +920,9 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 	{
 		fputs("<ReturnValue>\n", stream);
 		Indent(stream, indent + 4);
-		fprintf(stream, "<ReturnType>%s</ReturnType>\n",
-				CSTypeToName(returnType));
+		fputs("<ReturnType>", stream);
+		DumpString(CSTypeToName(returnType), stream);
+		fputs("</ReturnType>\n", stream);
 		Indent(stream, indent + 2);
 		fputs("</ReturnValue>\n", stream);
 	}
@@ -788,11 +944,11 @@ static void GenerateDocsForMethod(FILE *stream, ILNode_MethodDeclaration *decl,
 			if(paramInfo)
 			{
 				fputs("Name=\"", stream);
-				fputs(ILParameter_Name(paramInfo), stream);
+				DumpString(ILParameter_Name(paramInfo), stream);
 				fputs("\" ", stream);
 			}
 			fputs("Type=\"", stream);
-			fputs(CSTypeToName(ILTypeGetParam(signature, param)), stream);
+			DumpString(CSTypeToName(ILTypeGetParam(signature, param)), stream);
 			fputs("\"/>\n", stream);
 		}
 		Indent(stream, indent + 2);
@@ -851,7 +1007,7 @@ static void GenerateDocsForProperty(FILE *stream,
 	/* Output the member header */
 	Indent(stream, indent);
 	fputs("<Member MemberName=\"", stream);
-	fputs(ILProperty_Name(property), stream);
+	DumpString(ILProperty_Name(property), stream);
 	fputs("\">\n", stream);
 
 	/* Is this property an indexer? */
@@ -869,7 +1025,7 @@ static void GenerateDocsForProperty(FILE *stream,
 	ILDumpFlags(stream, ILProperty_Attrs(property),
 				ILPropertyDefinitionFlags, 0);
 	ILDumpMethodType(stream, ILProgramItem_Image(property), signature,
-			         0, 0, ILProperty_Name(property), getter);
+			         IL_DUMP_XML_QUOTING, 0, ILProperty_Name(property), getter);
 	fputs(" { ", stream);
 	if(getter)
 	{
@@ -877,7 +1033,7 @@ static void GenerateDocsForProperty(FILE *stream,
 		ILDumpFlags(stream, ILMethod_Attrs(getter), ILMethodDefinitionFlags, 0);
 		methodSig = ILMethod_Signature(getter);
 		ILDumpMethodType(stream, ILProgramItem_Image(getter), signature,
-				         0, 0, ILMethod_Name(getter), getter);
+				         IL_DUMP_XML_QUOTING, 0, ILMethod_Name(getter), getter);
 		putc(' ', stream);
 		ILDumpFlags(stream, ILMethod_ImplAttrs(getter),
 					ILMethodImplementationFlags, 0);
@@ -888,7 +1044,7 @@ static void GenerateDocsForProperty(FILE *stream,
 		ILDumpFlags(stream, ILMethod_Attrs(setter), ILMethodDefinitionFlags, 0);
 		methodSig = ILMethod_Signature(setter);
 		ILDumpMethodType(stream, ILProgramItem_Image(setter), signature,
-				         0, 0, ILMethod_Name(setter), setter);
+				         IL_DUMP_XML_QUOTING, 0, ILMethod_Name(setter), setter);
 		putc(' ', stream);
 		ILDumpFlags(stream, ILMethod_ImplAttrs(setter),
 					ILMethodImplementationFlags, 0);
@@ -899,7 +1055,7 @@ static void GenerateDocsForProperty(FILE *stream,
 	Indent(stream, indent + 2);
 	fputs("<MemberSignature Language=\"C#\" Value=\"", stream);
 	ILDumpFlags(stream, decl->modifiers, CSharpMethodFlags, 0);
-	fputs(CSTypeToName(ILTypeGetReturn(signature)), stream);
+	DumpString(CSTypeToName(ILTypeGetReturn(signature)), stream);
 	putc(' ', stream);
 	propertyName = ILProperty_Name(property);
 	if(isIndexer)
@@ -914,18 +1070,18 @@ static void GenerateDocsForProperty(FILE *stream,
 			int len = strlen(propertyName);
 			if(len > 5 && !strcmp(propertyName + len - 5, ".Item"))
 			{
-				fwrite(propertyName, 1, len - 4, stream);
+				DumpStringN(propertyName, len - 4, stream);
 				fputs("this", stream);
 			}
 			else
 			{
-				fputs(propertyName, stream);
+				DumpString(propertyName, stream);
 			}
 		}
 	}
 	else
 	{
-		fputs(propertyName, stream);
+		DumpString(propertyName, stream);
 	}
 	num = ILTypeNumParams(signature);
 	if(isIndexer)
@@ -937,14 +1093,14 @@ static void GenerateDocsForProperty(FILE *stream,
 			{
 				fputs(", ", stream);
 			}
-			fputs(CSTypeToName(ILTypeGetParam(signature, param)), stream);
+			DumpString(CSTypeToName(ILTypeGetParam(signature, param)), stream);
 			if(getter || setter)
 			{
 				paramInfo = FindMethodParam((getter ? getter : setter), param);
 				if(paramInfo)
 				{
 					putc(' ', stream);
-					fputs(ILParameter_Name(paramInfo), stream);
+					DumpString(ILParameter_Name(paramInfo), stream);
 				}
 			}
 		}
@@ -966,7 +1122,7 @@ static void GenerateDocsForProperty(FILE *stream,
 	fputs("<MemberType>Property</MemberType>\n", stream);
 
 	/* Dump the attributes for the property */
-	DumpAttributes(stream, decl->attributes, indent + 2);
+	DumpAttributes(stream, ILToProgramItem(property), indent + 2);
 
 	/* Output the property's type */
 	Indent(stream, indent + 2);
@@ -975,8 +1131,9 @@ static void GenerateDocsForProperty(FILE *stream,
 	{
 		fputs("<ReturnValue>\n", stream);
 		Indent(stream, indent + 4);
-		fprintf(stream, "<ReturnType>%s</ReturnType>\n",
-				CSTypeToName(returnType));
+		fputs("<ReturnType>", stream);
+		DumpString(CSTypeToName(returnType), stream);
+		fputs("</ReturnType>\n", stream);
 		Indent(stream, indent + 2);
 		fputs("</ReturnValue>\n", stream);
 	}
@@ -1000,12 +1157,12 @@ static void GenerateDocsForProperty(FILE *stream,
 				if(paramInfo)
 				{
 					fputs("Name=\"", stream);
-					fputs(ILParameter_Name(paramInfo), stream);
+					DumpString(ILParameter_Name(paramInfo), stream);
 					fputs("\" ", stream);
 				}
 			}
 			fputs("Type=\"", stream);
-			fputs(CSTypeToName(ILTypeGetParam(signature, param)), stream);
+			DumpString(CSTypeToName(ILTypeGetParam(signature, param)), stream);
 			fputs("\"/>\n", stream);
 		}
 		Indent(stream, indent + 2);
@@ -1027,6 +1184,86 @@ static void GenerateDocsForProperty(FILE *stream,
 	fputs("</Member>\n", stream);
 }
 
+/*
+ * Generate documentation for a specific event definition.
+ */
+static void GenerateDocsForEvent(FILE *stream, ILNode_EventDeclaration *decl,
+								 int indent)
+{
+	ILNode_ListIter iterator;
+	ILNode *eventDecl;
+	ILEvent *event;
+
+	/* Bail out if the field is private, and "-fprivate" is not supplied */
+	if(IsMemberPrivate(decl->modifiers))
+	{
+		if(!CCStringListContains(extension_flags, num_extension_flags,
+								 "private"))
+		{
+			return;
+		}
+	}
+
+	/* Scan all event declarators that are attached to the event definition */
+	ILNode_ListIter_Init(&iterator, decl->eventDeclarators);
+	while((eventDecl = ILNode_ListIter_Next(&iterator)) != 0)
+	{
+		/* Get the event descriptor */
+		event = ((ILNode_EventDeclarator *)eventDecl)->eventInfo;
+		if(!event)
+		{
+			continue;
+		}
+
+		/* Output the member header */
+		Indent(stream, indent);
+		fputs("<Member MemberName=\"", stream);
+		DumpString(ILEvent_Name(event), stream);
+		fputs("\">\n", stream);
+
+		/* Output the signature in ILASM form */
+		Indent(stream, indent + 2);
+		fputs("<MemberSignature Language=\"ILASM\" Value=\".event ", stream);
+		ILDumpFlags(stream, decl->modifiers, ILMethodDefinitionFlags, 0);
+		fputs("event ", stream);
+		DumpString(ILEvent_Name(event), stream);
+		/* TODO: add/remove methods */
+		fputs("\"/>\n", stream);
+
+		/* Output the signature in C# form */
+		Indent(stream, indent + 2);
+		fputs("<MemberSignature Language=\"C#\" Value=\"", stream);
+		ILDumpFlags(stream, decl->modifiers, CSharpMethodFlags, 0);
+		DumpString(CSTypeToName(ILEvent_Type(event)), stream);
+		putc(' ', stream);
+		DumpString(ILEvent_Name(event), stream);
+		/* TODO: add/remove methods */
+		fputs("\"/>\n", stream);
+
+		/* Output the member kind */
+		Indent(stream, indent + 2);
+		fputs("<MemberType>Event</MemberType>\n", stream);
+
+		/* Dump the attributes for the event */
+		DumpAttributes(stream, ILToProgramItem(event), indent + 2);
+
+		/* Events don't have return types or parameters */
+		Indent(stream, indent + 2);
+		fputs("<ReturnValue/>\n", stream);
+		Indent(stream, indent + 2);
+		fputs("<Parameters/>\n", stream);
+
+		/* Dump the doc comments for the event */
+		DumpDocComments(stream, decl->attributes, indent + 2);
+
+		/* Output the member footer */
+		Indent(stream, indent + 2);
+		fputs("<Excluded>0</Excluded>\n", stream);
+		Indent(stream, indent);
+		fputs("</Member>\n", stream);
+	}
+}
+
 /*
  * Generate documentation for a specific class definition and its members.
  */
@@ -1039,6 +1276,7 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 	ILNode_ListIter iterator;
 	ILNode *body;
 	ILNode *member;
+	int isEnum=0;
 
 	/* Bail out if this is a bad class definition */
 	if(!classInfo)
@@ -1061,7 +1299,9 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 
 	/* Output the type header */
 	Indent(stream, indent);
-	fprintf(stream, "<Type Name=\"%s\" FullName=\"", defn->name);
+	fputs("<Type Name=\"", stream);
+	DumpString(defn->name, stream);
+	fputs("\" FullName=\"", stream);
 	DumpClassName(stream, classInfo);
 	fputs("\" FullNameSP=\"", stream);
 	DumpClassNameSP(stream, classInfo);
@@ -1072,7 +1312,7 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 	Indent(stream, indent);
 	fputs("<TypeSignature Language=\"ILASM\" Value=\".class ", stream);
 	ILDumpFlags(stream, ILClass_Attrs(classInfo), ILTypeDefinitionFlags, 0);
-	fputs(defn->name, stream);
+	DumpString(defn->name, stream);
 	parent = ILClass_Parent(classInfo);
 	if(parent)
 	{
@@ -1092,15 +1332,31 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 	}
 	fputs("\"/>\n", stream);
 
-	/* Output the type signature, in C# form */
-	Indent(stream, indent);
-	fputs("<TypeSignature Language=\"C#\" Value=\"", stream);
-	ILDumpFlags(stream, defn->modifiers, CSharpTypeFlags, 0);
 	if(ILClassIsValueType(classInfo))
 	{
 		if(parent && !strcmp(ILClass_Name(parent), "Enum") &&
 		   ILClass_Namespace(parent) != 0 &&
 		   !strcmp(ILClass_Namespace(parent), "System"))
+		{
+			isEnum=1;
+		}
+	}
+
+	/* Output the type signature, in C# form */
+	Indent(stream, indent);
+	fputs("<TypeSignature Language=\"C#\" Value=\"", stream);
+	if(isEnum)
+	{
+		ILDumpFlags(stream,defn->modifiers & ~(IL_META_TYPEDEF_SEALED), 
+							CSharpTypeFlags,0);
+	}
+	else
+	{
+		ILDumpFlags(stream, defn->modifiers, CSharpTypeFlags, 0);
+	}
+	if(ILClassIsValueType(classInfo))
+	{
+		if(isEnum)
 		{
 			fputs("enum ", stream);
 		}
@@ -1117,8 +1373,8 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 	{
 		fputs("class ", stream);
 	}
-	fputs(ILClass_Name(classInfo), stream);
-	if(parent && !ILTypeIsObjectClass(ILClassToType(parent)))
+	DumpString(ILClass_Name(classInfo), stream);
+	if(!isEnum && parent && !ILTypeIsObjectClass(ILClassToType(parent)))
 	{
 		fputs(": ", stream);
 		DumpClassNameOther(stream, parent, classInfo);
@@ -1128,7 +1384,7 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 			fputs(", ", stream);
 		}
 	}
-	else
+	else if(!isEnum)
 	{
 		impl = ILClassNextImplements(classInfo, 0);
 		if(impl)
@@ -1136,7 +1392,7 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 			fputs(": ", stream);
 		}
 	}
-	if(impl)
+	if(!isEnum && impl)
 	{
 		DumpClassNameOther(stream, ILImplementsGetInterface(impl), classInfo);
 		while((impl = ILClassNextImplements(classInfo, impl)) != 0)
@@ -1211,7 +1467,7 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 	}
 
 	/* Dump the attributes for the type */
-	DumpAttributes(stream, defn->attributes, indent);
+	DumpAttributes(stream, ILToProgramItem(classInfo), indent);
 
 	/* Dump the class members */
 	Indent(stream, indent);
@@ -1246,12 +1502,12 @@ static void GenerateDocsForClass(FILE *stream, ILNode_ClassDefn *defn,
 								    (ILNode_PropertyDeclaration *)member,
 								    indent);
 		}
-#if 0
 		else if(yykind(member) == yykindof(ILNode_EventDeclaration))
 		{
-			/* TODO: document an event */
+			GenerateDocsForEvent(stream,
+								 (ILNode_EventDeclaration *)member,
+								 indent);
 		}
-#endif
 		else if(yykind(member) == yykindof(ILNode_ClassDefn))
 		{
 			GenerateDocsForClass(stream, (ILNode_ClassDefn *)member,
