@@ -101,16 +101,35 @@ extern	"C" {
 
 #ifdef IL_USE_JIT
 int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
+						int isCtor, void *_this,
 					    void *argBuffer, void **jitArgs, void *userData)
 {
 	VA_LIST va;
-	ILExecValue *args = (ILExecValue *)userData;
 	ILUInt32 param, numParams;
 	ILType *paramType;
 
 	/* Copy the incoming "va_list" value */
 	ILMemCpy(&va, userData, sizeof(VA_LIST));
 
+	/* Push the arguments onto the evaluation stack */
+	if(ILType_HasThis(signature) && !isCtor)
+	{
+		/* Push the "this" argument */
+		if(_this)
+		{
+			/* Use the supplied "this" parameter */
+			*jitArgs = argBuffer;
+			*((void **)argBuffer) = _this;
+		}
+		else
+		{
+			/* Get the "this" parameter from the varargs list */
+			*jitArgs = argBuffer;
+			*((void **)argBuffer) = (void *)(VA_ARG(va, ILObject *));
+		}
+		argBuffer += sizeof(void *);
+		++jitArgs;
+	}
 	numParams = ILTypeNumParams(signature);
 	for(param = 1; param <= numParams; ++param)
 	{
@@ -124,32 +143,62 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 
 				case IL_META_ELEMTYPE_BOOLEAN:
 				case IL_META_ELEMTYPE_I1:
+				{
+					*jitArgs = argBuffer;
+					*((ILInt8 *)argBuffer) = (ILInt8)VA_ARG(va, ILVaInt);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_I2:
+				{
+					*jitArgs = argBuffer;
+					*((ILInt16 *)argBuffer) = (ILInt16)VA_ARG(va, ILVaInt);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_I4:
 			#ifdef IL_NATIVE_INT32
 				case IL_META_ELEMTYPE_I:
 			#endif
 				{
 					*jitArgs = argBuffer;
-					*((ILVaInt *)argBuffer) = VA_ARG(va, ILVaInt);
-					argBuffer += sizeof(ILVaInt);
-					++args;
+					*((ILInt32 *)argBuffer) = (ILInt32)VA_ARG(va, ILVaInt);
+					argBuffer += sizeof(ILNativeFloat);
 					++jitArgs;
 				}
 				break;
 
 				case IL_META_ELEMTYPE_U1:
+				{
+					*jitArgs = argBuffer;
+					*((ILUInt8 *)argBuffer) = (ILUInt8)VA_ARG(va, ILVaUInt);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_U2:
 				case IL_META_ELEMTYPE_CHAR:
+				{
+					*jitArgs = argBuffer;
+					*((ILUInt16 *)argBuffer) = (ILUInt16)VA_ARG(va, ILVaUInt);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_U4:
 			#ifdef IL_NATIVE_INT32
 				case IL_META_ELEMTYPE_U:
 			#endif
 				{
 					*jitArgs = argBuffer;
-					*((ILVaUInt *)argBuffer) = VA_ARG(va, ILVaUInt);
-					argBuffer += sizeof(ILVaUInt);
-					++args;
+					*((ILUInt32 *)argBuffer) = (ILUInt32)VA_ARG(va, ILVaUInt);
+					argBuffer += sizeof(ILNativeFloat);
 					++jitArgs;
 				}
 				break;
@@ -157,13 +206,11 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 				case IL_META_ELEMTYPE_I8:
 			#ifdef IL_NATIVE_INT64
 				case IL_META_ELEMTYPE_I:
-				case IL_META_ELEMTYPE_U:
 			#endif
 				{
 					*jitArgs = argBuffer;
 					*((ILInt64 *)argBuffer) = VA_ARG(va, ILInt64);
-					argBuffer += sizeof(ILInt64);
-					++args;
+					argBuffer += sizeof(ILNativeFloat);
 					++jitArgs;
 				}
 				break;
@@ -176,19 +223,33 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 					*jitArgs = argBuffer;
 					*((ILUInt64 *)argBuffer) = VA_ARG(va, ILUInt64);
 					argBuffer += sizeof(ILUInt64);
-					++args;
 					++jitArgs;
 				}
 				break;
 
 				case IL_META_ELEMTYPE_R4:
+				{
+					*jitArgs = argBuffer;
+					*((ILFloat *)argBuffer) = (ILFloat)VA_ARG(va, ILVaDouble);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_R8:
+				{
+					*jitArgs = argBuffer;
+					*((ILDouble *)argBuffer) = (ILDouble)VA_ARG(va, ILVaDouble);
+					argBuffer += sizeof(ILNativeFloat);
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_R:
 				{
 					*jitArgs = argBuffer;
-					*((ILVaDouble *)argBuffer) = VA_ARG(va, ILVaDouble);
-					argBuffer += sizeof(ILUInt64);
-					++args;
+					*((ILNativeFloat *)argBuffer) = (ILNativeFloat)VA_ARG(va, ILVaDouble);
+					argBuffer += sizeof(ILNativeFloat);
 					++jitArgs;
 				}
 				break;
@@ -199,8 +260,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 					   as a pointer to a temporary typedref structure */
 					*jitArgs = argBuffer;
 					*((void **)argBuffer) = VA_ARG(va, void *);
-					argBuffer += sizeof(void *);
-					++args;
+					argBuffer += sizeof(ILNativeFloat);
 					++jitArgs;
 				}
 				break;
@@ -211,8 +271,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 			/* Process an object reference */
 			*jitArgs = argBuffer;
 			*((ILObject **)argBuffer) = VA_ARG(va, ILObject *);
-			argBuffer += sizeof(ILObject *);
-			++args;
+			argBuffer += sizeof(ILNativeFloat);
 			++jitArgs;
 		}
 		else if(ILType_IsValueType(paramType))
@@ -221,7 +280,6 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 			   put the value into a temporary location and then
 			   passed a pointer to the temporary to us */
 			*jitArgs = (void *)(VA_ARG(va, void *));
-			++args;
 			++jitArgs;
 		}
 		else if(paramType != 0 && ILType_IsComplex(paramType) &&
@@ -230,8 +288,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 			/* Process a value that is being passed by reference */
 			*jitArgs = argBuffer;
 			*((void **)argBuffer) = VA_ARG(va, void *);
-			argBuffer += sizeof(void *);
-			++args;
+			argBuffer += sizeof(ILNativeFloat);
 			++jitArgs;
 		}
 		else
@@ -239,8 +296,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
 			/* Assume that everything else is an object reference */
 			*jitArgs = argBuffer;
 			*((ILObject **)argBuffer) = VA_ARG(va, ILObject *);
-			argBuffer += sizeof(ILObject *);
-			++args;
+			argBuffer += sizeof(ILNativeFloat);
 			++jitArgs;
 		}
 	}
@@ -405,6 +461,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILMethod *method,
 
 #ifdef IL_USE_JIT
 int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
+					   int isCtor, void *_this,
 					   void *argBuffer, void **jitArgs, void *userData)
 {
 	ILExecValue *args = (ILExecValue *)userData;
@@ -412,6 +469,24 @@ int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
 	ILType *paramType;
 
 	/* Store pointers to the args in the jitArgs Array. */
+	if(ILType_HasThis(signature) && !isCtor)
+	{
+		/* Push the "this" argument */
+		if(_this)
+		{
+			/* Use the supplied "this" parameter */
+			*jitArgs = argBuffer;
+			*((void **)argBuffer) = _this;
+			argBuffer += sizeof(ILNativeFloat);
+		}
+		else
+		{
+			/* Get the "this" parameter from the argument list */
+			*jitArgs = &(args->objValue);
+			++args;
+		}
+		++jitArgs;
+	}
 	numParams = ILTypeNumParams(signature);
 	for(param = 1; param <= numParams; ++param)
 	{
@@ -425,8 +500,26 @@ int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
 
 				case IL_META_ELEMTYPE_BOOLEAN:
 				case IL_META_ELEMTYPE_I1:
+				{
+					*jitArgs = argBuffer;
+					*((ILInt8 *)argBuffer) = (ILInt8)(args->int32Value);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_I2:
 				case IL_META_ELEMTYPE_CHAR:
+				{
+					*jitArgs = argBuffer;
+					*((ILInt16 *)argBuffer) = (ILInt16)(args->int32Value);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_I4:
 			#ifdef IL_NATIVE_INT32
 				case IL_META_ELEMTYPE_I:
@@ -439,7 +532,25 @@ int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
 				break;
 
 				case IL_META_ELEMTYPE_U1:
+				{
+					*jitArgs = argBuffer;
+					*((ILUInt8 *)argBuffer) = (ILUInt8)(args->uint32Value);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_U2:
+				{
+					*jitArgs = argBuffer;
+					*((ILUInt16 *)argBuffer) = (ILUInt16)(args->uint32Value);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_U4:
 			#ifdef IL_NATIVE_INT32
 				case IL_META_ELEMTYPE_U:
@@ -474,7 +585,25 @@ int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
 				break;
 
 				case IL_META_ELEMTYPE_R4:
+				{
+					*jitArgs = argBuffer;
+					*((ILFloat *)argBuffer) = (ILFloat)(args->floatValue);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_R8:
+				{
+					*jitArgs = argBuffer;
+					*((ILDouble *)argBuffer) = (ILDouble)(args->floatValue);
+					argBuffer += sizeof(ILNativeFloat);
+					++args;
+					++jitArgs;
+				}
+				break;
+
 				case IL_META_ELEMTYPE_R:
 				{
 					*jitArgs = &(args->floatValue);
@@ -686,6 +815,14 @@ int _ILCallPackVParams(ILExecThread *thread, ILMethod *method,
 }
 #endif
 
+#ifdef IL_USE_JIT
+void _ILCallUnpackDirectResult(ILExecThread *thread, ILMethod *method,
+					           int isCtor, void *result, void *userData)
+{
+	/* This is a NOOP for now. */
+}
+
+#else
 void _ILCallUnpackDirectResult(ILExecThread *thread, ILMethod *method,
 					           int isCtor, void *result, void *userData)
 {
@@ -817,7 +954,15 @@ void _ILCallUnpackDirectResult(ILExecThread *thread, ILMethod *method,
 		}
 	}
 }
+#endif
 
+#ifdef IL_USE_JIT
+void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
+				          int isCtor, void *_result, void *userData)
+{
+	/* This is a NOOP for now. */
+}
+#else
 void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
 				          int isCtor, void *_result, void *userData)
 {
@@ -918,6 +1063,7 @@ void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
 		}
 	}
 }
+#endif
 
 #ifdef IL_USE_JIT
 int _ILCallMethod(ILExecThread *thread, ILMethod *method,
@@ -927,10 +1073,16 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 {
 	ILType *signature = ILMethod_Signature(method);
 	ILUInt32 numParams = ILTypeNumParams(signature);
+#ifdef IL_JIT_THREAD_IN_SIGNATURE
 	/* We need an additional parameter for the ILExecThread. */
 	ILUInt32 totalParams = numParams + 1;
 	/* current arg in the parameter Array. */
 	ILUInt32 current = 1;
+#else
+	ILUInt32 totalParams = numParams;
+	/* current arg in the parameter Array. */
+	ILUInt32 current = 0;
+#endif
 	/* Some infos that we'll need later. */
 	ILClass *info = ILMethod_Owner(method);
 	/* Flag if this is an array or string constructor. */
@@ -947,14 +1099,15 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 		}
 		else
 		{
-			ILType *type = ILType_FromClass(info);
-			ILType *synType = type = ILClassGetSynType(info);
+			ILType *type = ILClassToType(info);
+			ILType *synType = ILClassGetSynType(info);
 
 			if(!(synType && ILType_IsArray(synType)) && !ILTypeIsStringClass(type))
 			{
 				/* We need an additional parameter for the this pointer */
 				/* for all casese except arrays or strings. */
 				totalParams++;
+				numParams++;
 			}
 			else
 			{
@@ -969,8 +1122,10 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 	/* We take the biggest possible arg type here so we don't need to compute it. */
  	ILNativeFloat jitArgsBuffer[numParams];
 
+#ifdef IL_JIT_THREAD_IN_SIGNATURE
 	/* Store the ILExecThread instance in arg[0]. */
 	jitArgs[0] = &thread;
+#endif
 
 	if(ILType_HasThis(signature))
 	{
@@ -979,28 +1134,47 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 			if(!isArrayOrString)
 			{
 				/* We need to allocate the Object. */
-				if(!(_this = _ILEngineAlloc(thread, ILMethod_Owner(method), 0)))
+				ILUInt32 alignment;
+				ILUInt32 size = _ILLayoutClassReturn(_ILExecThreadProcess(thread), info, &alignment);
+
+				if(!(_this = _ILEngineAlloc(thread, ILMethod_Owner(method), size)))
 				{
 					return 1;
 				}
+			#ifdef IL_JIT_THREAD_IN_SIGNATURE
 				jitArgs[1] = &_this;
+			#else
+				jitArgs[0] = &_this;
+			#endif
 				current++;
 			}
 		}
-		else
-		{
-			jitArgs[1] = &_this;
-			current++;
-		}
 	}
 
-	if((*pack)(thread, signature, &(jitArgsBuffer[0]), &(jitArgs[current]), userData))
+	if((*pack)(thread, signature, isCtor, _this, 
+			   &(jitArgsBuffer[0]), &(jitArgs[current]), userData))
 	{
 		return 1;
 	}
 
 	/* Now we can call the jitted function. */
-	return ILJitCallMethod(thread, method, jitArgs, result);
+	if(ILJitCallMethod(thread, method, jitArgs, result))
+	{
+		if(isCtor && !isArrayOrString && result)
+		{
+			/* Return the this pointer. */
+		#ifdef IL_JIT_THREAD_IN_SIGNATURE
+			*(void **)result = *(void **)(jitArgs[1]);
+		#else
+			*(void **)result = *(void **)(jitArgs[0]);
+		#endif
+		}
+		return 0;
+	}
+	/* If we get here there is an unhandled exception. */
+	/* We need to set the thread->thrownException. */
+	thread->thrownException = thread->currentException;
+	return 1;
 }
 #else
 int _ILCallMethod(ILExecThread *thread, ILMethod *method,
