@@ -22,6 +22,9 @@
 #include "il_coder.h"
 #include "il_opcodes.h"
 #include "il_utils.h"
+#ifdef IL_CONFIG_DEBUGGER
+#include "debugger.h"
+#endif
 #ifndef IL_WITHOUT_TOOLS
 #include "il_dumpasm.h"
 #endif
@@ -29,10 +32,13 @@
 #include "lib_defs.h"
 #include "method_cache.h"
 #include "cvm_config.h"
+#include "cctormgr.h"
 #if defined(HAVE_LIBFFI)
 #include "ffi.h"
 #endif
 
+#ifdef IL_USE_CVM
+
 #ifdef	__cplusplus
 extern	"C" {
 #endif
@@ -76,12 +82,17 @@ struct _tagILCVMCoder
 	ILCVMLabel     *labelList;
 	int				labelOutOfMemory;
 	unsigned char  *switchStart;
-	ILMethod	   *currentMethod;
 	int				debugEnabled;
+#ifdef IL_CONFIG_DEBUGGER
+	/* Flag if current method can be debugged */
+	int markBreakpoints;
+#endif
 	int				flags;
 	long			nativeArgPosn;
 	long			nativeArgHeight;
 	ILExecProcess  *process;		/* Backpointer to the owning process */
+	/* The manager for running the required cctors. */
+	ILCCtorMgr		cctorMgr;
 };
 
 /*
@@ -151,6 +162,12 @@ static ILCoder *CVMCoder_Create(ILExecProcess *process, ILUInt32 size,
 		ILFree(coder);
 		return 0;
 	}
+	if(!ILCCtorMgr_Init(&(coder->cctorMgr), 10))
+	{
+		ILCacheDestroy(coder->cache);
+		ILFree(coder);
+		return 0;
+	}
 	coder->start = 0;
 	coder->stackCheck = 0;
 	coder->needTry = 0;
@@ -167,7 +184,6 @@ static ILCoder *CVMCoder_Create(ILExecProcess *process, ILUInt32 size,
 	coder->labelList = 0;
 	coder->labelOutOfMemory = 0;
 	coder->switchStart = 0;
-	coder->currentMethod = 0;
 	coder->debugEnabled = 0;
 	coder->flags = 0;
 	coder->nativeArgPosn = 0;
@@ -221,6 +237,7 @@ static void CVMCoder_Destroy(ILCoder *_coder)
 {
 	ILCVMCoder *coder = (ILCVMCoder *)_coder;
 	ILCacheDestroy(coder->cache);
+	ILCCtorMgr_Destroy(&(coder->cctorMgr));
 	if(coder->argOffsets)
 	{
 		ILFree(coder->argOffsets);
@@ -259,10 +276,40 @@ static ILUInt32 CVMCoder_GetNativeOffset(ILCoder *_coder, void *start,
 static void CVMCoder_MarkBytecode(ILCoder *coder, ILUInt32 offset)
 {
 	ILCacheMarkBytecode(&(((ILCVMCoder *)coder)->codePosn), offset);
-	if(((ILCVMCoder *)coder)->debugEnabled)
+#ifdef IL_CONFIG_DEBUGGER
+	/* Insert potential breakpoint */
+	if(((ILCVMCoder *)coder)->markBreakpoints)
 	{
 		CVM_OUT_BREAK(IL_BREAK_DEBUG_LINE);
 	}
+#endif
+}
+
+/*
+ * Run the class initializers queued during generation of the last method.
+ */
+static ILInt32 CVMCoder_RunCCtors(ILCoder *_coder, void *userData)
+{
+	ILCVMCoder *coder = (ILCVMCoder *)_coder;
+	return ILCCtorMgr_RunCCtors(&(coder->cctorMgr), userData);
+}
+
+/*
+ * Run the class initializer for the given class.
+ */
+static ILInt32 CVMCoder_RunCCtor(ILCoder *_coder, ILClass *classInfo)
+{
+	ILCVMCoder *coder = (ILCVMCoder *)_coder;
+	return ILCCtorMgr_RunCCtor(&(coder->cctorMgr), classInfo);
+}
+
+/*
+ * Handle the method lock while running class initializers.
+ */
+static void *CVMCoder_HandleLockedMethod(ILCoder *_coder, ILMethod *method)
+{
+	ILCVMCoder *coder = (ILCVMCoder *)_coder;
+	return ILCCtorMgr_HandleLockedMethod(&(coder->cctorMgr), method);
 }
 
 /*
@@ -473,9 +520,15 @@ ILCoderClass const _ILCVMCoderClass =
 	CVMCoder_CheckNull,
 	CVMCoder_Convert,
 	CVMCoder_ConvertCustom,
+	CVMCoder_RunCCtors,
+	CVMCoder_RunCCtor,
+	CVMCoder_HandleLockedMethod,
 	"sentinel"
 };
 
 #ifdef	__cplusplus
 };
 #endif
+
+#endif /* IL_USE_CVM */
+
