@@ -279,14 +279,6 @@ static int CVMEntryGen(CVMEntryContext *ctx, ILCVMCoder *coder,
 	/* Set the number of arguments, which initialize's the method's frame */
 	CVM_OUT_WIDE(COP_SET_NUM_ARGS, ctx->numArgWords);
 
-	/* Is this a static constructor? */
-	if(ILMethod_IsStaticConstructor(method))
-	{
-		/* Output a "cctor_once" instruction to ensure that this
-		   method's body can only be executed once */
-		CVM_OUT_NONE(COP_CCTOR_ONCE);
-	}
-
 	/* If this is a constructor, then back-patch the push down size,
 	   which is one less than the number of argument words */
 	if(isConstructor)
@@ -1261,6 +1253,9 @@ static int CVMCoder_Setup(ILCoder *_coder, unsigned char **start,
 	ILCVMCoder *coder = ((ILCVMCoder *)_coder);
 	ILType *signature = ILMethod_Signature(method);
 	CVMEntryContext ctx;
+#ifdef IL_CONFIG_DEBUGGER
+	ILDebugger *debugger;
+#endif
 
 	/* Initialize the entry point context */
 	CVMEntryInit(&ctx);
@@ -1277,6 +1272,15 @@ static int CVMCoder_Setup(ILCoder *_coder, unsigned char **start,
 		return 0;
 	}
 
+#ifdef IL_CONFIG_DEBUGGER
+	/* Check if this method can be debugged */
+	debugger = ILDebuggerFromProcess(coder->process);
+	coder->markBreakpoints = (debugger && ILDebuggerIsAssemblyWatched(debugger, method));
+#endif
+
+	/* Set the current method in the coder and queue a cctor if needed. */
+	ILCCtorMgr_SetCurrentMethod(&(coder->cctorMgr), method);
+
 	/* Generate the entry point code */
 	return CVMEntryGen(&ctx, coder, method, signature,
 					   ILMethod_IsConstructor(method), 1, start, 1);
@@ -1367,6 +1371,9 @@ static int CVMCoder_SetupExtern(ILCoder *_coder, unsigned char **start,
 		return 0;
 	}
 
+	/* Set the current method in the coder and queue a cctor if needed. */
+	ILCCtorMgr_SetCurrentMethod(&(coder->cctorMgr), method);
+
 	/* Output the body of the method */
 	CVMEntryPushLeadIn(&ctx, coder, signature, useRawCalls, isInternal, 0);
 	CVMEntryPushNativeArgs(&ctx, coder, method, signature,
