@@ -25,6 +25,7 @@
 #include "il_system.h"
 #include "il_program.h"
 #include "il_coder.h"
+#include "il_debugger.h"
 #include "il_align.h"
 #include "il_gc.h"
 #include "il_utils.h"
@@ -296,6 +297,13 @@ struct _tagILExecProcess
 
 #endif /* IL_CONFIG_DEBUG_LINES */
 
+#ifdef IL_CONFIG_DEBUGGER
+
+	/* Debugger attached to this process */
+	ILDebugger		*debugger;
+
+#endif
+
 	/* Cryptographic seed material */
 	ILMutex        	   *randomLock;
 	int					randomBytesDelivered;
@@ -327,6 +335,19 @@ struct _tagILExecProcess
 #endif /* IL_CONFIG_APPDOMAINS */
 };
 
+#ifdef IL_CONFIG_DEBUGGER
+/*
+ * Information about local variable or function parameter.
+ */
+typedef struct _tagILWatch
+{
+	void           *addr;			/* Address of variable */
+	int				flag;
+	void		   *frame;			/* Frame pointer */
+
+} ILWatch;
+#endif
+
 /*
  * Information that is stored in a stack call frame.
  * Offsets are used to refer to the stack instead of
@@ -371,7 +392,8 @@ struct _tagILExecThread
 	CVMWord		   *frame;			/* Base of the local variable frame */
 	CVMWord        *stackTop;		/* Current stack top */
 	ILMethod       *method;			/* Current method being executed */
-	
+	ILUInt32		offset;			/* Current IL offset */
+
 	/* Last exception that was thrown */
 	ILObject       *thrownException;
 
@@ -400,6 +422,13 @@ struct _tagILExecThread
 	ILUInt32		numFrames;
 	ILUInt32		maxFrames;
 
+#ifdef IL_CONFIG_DEBUGGER
+	/* Stack for watching local variables and function params */
+	ILWatch	       *watchStack;
+	ILUInt32		numWatches;
+	ILUInt32		maxWatches;
+#endif
+
 	/* Thread-static values for this thread */
 	void		  **threadStaticSlots;
 	ILUInt32		threadStaticSlotsUsed;
@@ -602,6 +631,7 @@ extern ILCoderClass const _ILCVMCoderClass;
  */
 #ifdef IL_USE_JIT
 int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
+						int isCtor, void *_this,
 					    void *argBuffer, void **jitArgs, void *userData);
 #else
 int _ILCallPackVaParams(ILExecThread *thread, ILMethod *method,
@@ -614,6 +644,7 @@ int _ILCallPackVaParams(ILExecThread *thread, ILMethod *method,
  */
 #ifdef IL_USE_JIT
 int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
+					   int isCtor, void *_this,
 					   void *argBuffer, void **jitArgs, void *userData);
 #else
 int _ILCallPackVParams(ILExecThread *thread, ILMethod *method,
@@ -639,6 +670,7 @@ void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
  */
 #ifdef IL_USE_JIT
 typedef int (*ILCallPackFunc)(ILExecThread *thread, ILType *signature,
+							  int isCtor, void *_this,
 					          void *argBuffer, void **jitArgs, void *userData);
 #else
 typedef int (*ILCallPackFunc)(ILExecThread *thread, ILMethod *method,
@@ -682,6 +714,23 @@ ILUInt32 _ILLayoutClassReturn(ILExecProcess *process, ILClass *info,
  */
 int _ILLayoutAlreadyDone(ILClass *info);
 
+#ifdef	IL_USE_TYPED_ALLOCATION
+/*
+ * Build the type descriptor used by the garbage collector to check which words
+ * in the object have to be scanned.
+ */
+int ILGCBuildTypeDescriptor(ILClassPrivate *classPrivate,
+							ILInt32 isManagedInstance);
+
+/*
+ * Build the type descriptor used by the garbage collector to check which words
+ * in the object have to be scanned for the block holding the static fields of
+ * the class..
+ */
+ILNativeUInt ILGCBuildStaticTypeDescriptor(ILClass *classInfo,
+										   ILInt32 isManagedStatic);
+#endif	/* IL_USE_TYPED_ALLOCATION */
+
 /*
  * Verify the contents of a method.
  */
@@ -780,6 +829,13 @@ int _ILGetInternalArray(ILMethod *method, int *isCtor, ILInternalInfo *info);
  */
 int _ILGetInternalDelegate(ILMethod *method, int *isCtor,  ILInternalInfo *info);
 
+/*
+ * Look up a class name that is length-specified.
+ */
+ILClass *_ILLookupClass(ILExecProcess *process,
+						const char *className,
+						int classNameLen);
+
 /*
  * Look up an interface method.  Returns NULL if not found.
  */
@@ -841,6 +897,14 @@ ILCallFrame *_ILGetNextCallFrame(ILExecThread *thread, ILCallFrame *frame);
  */
 ILCallFrame *_ILAllocCallFrame(ILExecThread *thread);
 
+#ifdef IL_CONFIG_DEBUGGER
+/*
+ * Reallocate the watches for a given thread in order
+ * to provide more space.  Returns NULL if out of memory.
+ */
+ILWatch *_ILAllocWatch(ILExecThread *thread);
+#endif
+
 /*
  * Set a reference type field of a an object instance.
  */
@@ -848,6 +912,11 @@ int _ILSystemObjectSetField(ILExecThread *thread, ILObject* obj,
 							   const char *fieldName, const char *signature,
 							   ILObject *value);
 
+/*
+ * Set the stack trace for an exception to the current call context.
+ */
+void _ILSetExceptionStackTrace(ILExecThread *thread, ILObject *object);
+
 /*
  * Create a system exception object of a particular class.
  * We do this very carefully, to avoid re-entering the engine.
