@@ -117,7 +117,7 @@ void _IL_AsyncResult_SetOutParams(ILExecThread *_thread, ILObject *del,
 
 	paramcount = ILTypeNumParams(invokeSignature);
 
-	if (args->length < paramcount || outParams->length < paramcount)
+	if (ArrayLength(args) < paramcount || ArrayLength(outParams) < paramcount)
 	{
 		return;
 	}
@@ -136,7 +136,7 @@ void _IL_AsyncResult_SetOutParams(ILExecThread *_thread, ILObject *del,
 		{
 			paramType = ILType_Ref(paramType);
 
-			if (j >= outParams->length)
+			if (j >= ArrayLength(outParams))
 			{
 				break;
 			}
@@ -175,6 +175,7 @@ static ILObject *Delegate_ctor(ILExecThread *thread,
 	return _this;
 }
 
+#ifdef IL_USE_CVM
 /*
  * Parameter information for delegate invocation.
  */
@@ -554,6 +555,38 @@ static void Delegate_EndInvoke(ILExecThread *thread,
 	thread->stackTop = stackTop;
 }
 
+#endif /* IL_USE_CVM */
+
+#ifdef IL_USE_JIT
+
+/*
+ * public Type Invoke(...);
+ */
+static void Delegate_Invoke(ILExecThread *thread,
+							void *result,
+							ILObject *_this)
+{
+	/* This is a dummy because the real function is generated by the */
+	/* jit coder. */
+}
+
+static ILObject *Delegate_BeginInvoke(ILExecThread *thread, ILObject *_this)
+{	
+	/* This is a dummy because the real function is generated by the */
+	/* jit coder. */
+	return 0;
+}
+
+static void Delegate_EndInvoke(ILExecThread *thread,
+							void *result,
+							ILObject *_this)
+{
+	/* This is a dummy because the real function is generated by the */
+	/* jit coder. */
+}
+
+#endif /* IL_USE_JIT */
+
 #if !defined(HAVE_LIBFFI)
 
 /*
