@@ -158,33 +158,35 @@ int _ILIsBreak(ILExecThread *thread, ILMethod *method)
 
 void _ILBreak(ILExecThread *thread, int type)
 {
-	unsigned char *start;
-	ILInt32 offset;
 	int action;
+#ifdef IL_USE_CVM
+	unsigned char *start;
+#endif
 
 	if(thread->process->debugHookFunc)
 	{
+#ifdef IL_USE_CVM
 		/* Consult the coder to convert the PC into an IL offset */
 		if(thread->method && thread->pc != IL_INVALID_PC)
 		{
-			start = (unsigned char *)ILMethodGetUserData(thread->method);
-			if(ILMethodIsConstructor(thread->method))
-			{
-				start -= ILCoderCtorOffset(thread->process->coder);
-			}
-			offset = (ILInt32)(ILCoderGetILOffset
-					(thread->process->coder, (void *)start,
-					 (ILUInt32)(thread->pc - start), 0));
+				start = (unsigned char *)ILMethodGetUserData(thread->method);
+				if(ILMethodIsConstructor(thread->method))
+				{
+						start -= ILCoderCtorOffset(thread->process->coder);
+				}
+				thread->offset = (ILInt32)(ILCoderGetILOffset
+								(thread->process->coder, (void *)start,
+								(ILUInt32)(thread->pc - start), 0));
 		}
 		else
 		{
-			offset = -1;
+				thread->offset = -1;
 		}
-
+#endif
 		/* Call the debugger to process the breakpoint */
 		action = (*(thread->process->debugHookFunc))
 			(thread->process->debugHookData,
-			 thread, thread->method, offset, type);
+			 thread, thread->method, thread->offset, type);
 
 		/* Abort the engine if requested by the debugger */
 		if(action == IL_HOOK_ABORT)
@@ -269,12 +271,17 @@ void _IL_Debugger_Log(ILExecThread *thread, ILInt32 level,
 ILInt32 _IL_StackFrame_InternalGetTotalFrames(ILExecThread *thread)
 {
 	ILInt32 num = 0;
+#ifdef IL_USE_CVM
 	ILCallFrame *frame = _ILGetCallFrame(thread, 0);
 	while(frame != 0)
 	{
 		++num;
 		frame = _ILGetNextCallFrame(thread, frame);
 	}
+#endif
+#ifdef IL_USE_JIT
+	num = _ILJitDiagNumFrames(thread);
+#endif
 	return num;
 }
 
@@ -284,6 +291,7 @@ ILInt32 _IL_StackFrame_InternalGetTotalFrames(ILExecThread *thread)
 void _IL_StackFrame_InternalGetMethod(ILExecThread *thread,
 									  void *result, ILInt32 skipFrames)
 {
+#ifdef IL_USE_CVM
 	ILCallFrame *frame = _ILGetCallFrame(thread, skipFrames);
 	if(frame)
 	{
@@ -293,6 +301,10 @@ void _IL_StackFrame_InternalGetMethod(ILExecThread *thread,
 	{
 		*((ILMethod **)result) = 0;
 	}
+#endif
+#ifdef IL_USE_JIT
+	*(ILMethod **)result = _ILJitGetCallingMethod(thread, skipFrames);
+#endif
 }
 
 /*
@@ -365,7 +377,11 @@ ILString *_IL_StackFrame_InternalGetDebugInfo
 	*col = 0;
 
 	/* Validate the method handle */
+#ifdef IL_USE_JIT
+	method = (ILMethod *)_method;
+#else
 	method = *((ILMethod **)_method);
+#endif
 	if(!method)
 	{
 		return 0;
@@ -403,6 +419,7 @@ ILString *_IL_StackFrame_InternalGetDebugInfo
  */
 System_Array *_IL_StackFrame_GetExceptionStackTrace(ILExecThread *thread)
 {
+#ifdef IL_USE_CVM
 #ifdef IL_CONFIG_REFLECTION
 	ILInt32 num;
 	ILCallFrame *frame;
@@ -446,7 +463,7 @@ System_Array *_IL_StackFrame_GetExceptionStackTrace(ILExecThread *thread)
 	{
 		return 0;
 	}
-	((System_Array *)array)->length = num;
+	ArrayLength(array) = num;
 
 	/* Fill the array with the packed stack data */
 	data = (PackedStackFrame *)ArrayToBuffer(array);
@@ -489,6 +506,10 @@ System_Array *_IL_StackFrame_GetExceptionStackTrace(ILExecThread *thread)
 #else
 	return 0;
 #endif
+#endif
+#ifdef IL_USE_JIT
+	return _ILJitGetExceptionStackTrace(thread);
+#endif
 }
 
 #else /* !IL_CONFIG_DEBUG_LINES */
