@@ -485,7 +485,7 @@ ILNativeInt _IL_Marshal_UnsafeAddrOfPinnedArrayElement(ILExecThread *_thread,
 
 	if(UnmanagedOK(_thread) && arr && index >= 0 &&
 	   _ILIsSArray((System_Array *)arr) &&
-	   index < ((System_Array *)arr)->length)
+	   index < ArrayLength(arr))
 	{
 		type = ILClassGetSynType(GetObjectClass(arr));
 		type = ILTypeGetEnumType(ILType_ElemType(type));
@@ -635,15 +635,20 @@ void _ILStructToNative(ILExecThread *thread, void *value, ILType *type)
 		type = ILField_Type(field);
 		ptr = (void *)(((unsigned char *)value) + field->offset);
 		_ILStructToNative(thread, ptr, type);
-	    marshalType = ILPInvokeGetMarshalType(pinv, method, 0,
+		marshalType = ILPInvokeGetMarshalType(pinv, method, 0,
 	   										  &customName, &customNameLen,
 											  &customCookie,
 											  &customCookieLen, type);
 		/* TODO: convert other kinds of fields, not just delegates */
 		if(marshalType == IL_META_MARSHAL_FNPTR)
 		{
+	    #ifdef IL_USE_JIT
+			*((void **)ptr) = ILJitDelegateGetClosure
+				(*((ILObject **)ptr), type);
+	    #else
 			*((void **)ptr) = _ILDelegateGetClosure
 				(thread, *((ILObject **)ptr));
+	    #endif
 		}
 	}
 }
