@@ -861,11 +861,19 @@ System_Array *_IL_AppDomain_GetAssemblies(ILExecThread *thread,
  */
 ILObject *_IL_Assembly_GetCallingAssembly(ILExecThread *thread)
 {
+#ifdef IL_USE_JIT
+	ILMethod *method = _ILJitGetCallingMethod(thread, 1);
+	if(method)
+	{
+		return ImageToAssembly(thread, ILProgramItem_Image(method));
+	}
+#else
 	ILCallFrame *frame = _ILGetCallFrame(thread, 1);
 	if(frame && frame->method)
 	{
 		return ImageToAssembly(thread, ILProgramItem_Image(frame->method));
 	}
+#endif
 	else
 	{
 		return 0;
@@ -893,11 +901,19 @@ ILObject *_IL_Assembly_GetEntryAssembly(ILExecThread *thread)
  */
 ILObject *_IL_Assembly_GetExecutingAssembly(ILExecThread *thread)
 {
+#ifdef IL_USE_JIT
+	ILMethod *method = _ILJitGetCallingMethod(thread, 0);
+	if(method)
+	{
+		return ImageToAssembly(thread, ILProgramItem_Image(method));
+	}
+#else
 	ILCallFrame *frame = _ILGetCallFrame(thread, 0);
 	if(frame && frame->method)
 	{
 		return ImageToAssembly(thread, ILProgramItem_Image(frame->method));
 	}
+#endif
 	else
 	{
 		return 0;
@@ -1446,6 +1462,7 @@ ILObject *_IL_Assembly_LoadFromName(ILExecThread *thread,
 										image, &newImage);
 		if(loadError == 0)
 		{
+			*error = LoadError_OK;
 			return ImageToAssembly(thread, newImage);
 		}
 		else if(loadError == -1)
@@ -1564,7 +1581,7 @@ ILObject *_IL_Assembly_LoadFromBytes(ILExecThread *thread,
 	   to be able to modify the IL binary after we have loaded it.
 	   Or worse, have the garbage collector throw the "bytes" array away */
 	loadError = ILImageLoadFromMemory(ArrayToBuffer(bytes),
-									  (unsigned long)(long)(bytes->length),
+									  (unsigned long)(long)(ArrayLength(bytes)),
 									  thread->process->context,
 									  &image, IL_LOADFLAG_FORCE_32BIT, 0);
 	if(loadError == 0)
@@ -2300,7 +2317,11 @@ ILObject *_IL_FieldInfo_GetFieldFromHandle(ILExecThread *thread,
 									       void *handle)
 {
 	ILField *field;
+#ifdef IL_USE_JIT
+	field = ILProgramItemToField((ILProgramItem *)handle);
+#else
 	field = ILProgramItemToField((ILProgramItem *)(*((void **)handle)));
+#endif
 	if(field)
 	{
 		return _ILClrToObject(thread, field, "System.Reflection.ClrField");
@@ -2362,6 +2383,7 @@ ILObject* UnpackConstant(ILExecThread *thread,ILConstant* constant,
  			longValue = IL_READ_INT64(ptr);
  			return ILExecThreadBox(thread,type,&(longValue));
 
+#ifdef IL_CONFIG_FP_SUPPORTED
  		case IL_META_ELEMTYPE_R4:
 			if(len < 4)
 				return 0;
@@ -2374,6 +2396,15 @@ ILObject* UnpackConstant(ILExecThread *thread,ILConstant* constant,
  			doubleValue =  IL_READ_DOUBLE(ptr);
  			return ILExecThreadBox(thread,type,&(doubleValue));
 
+#else /* !IL_CONFIG_FP_SUPPORTED */
+
+		case IL_META_ELEMTYPE_R4:
+		case IL_META_ELEMTYPE_R8:
+			ILExecThreadThrowSystem(thread,
+					"System.NotImplementedException", 0);
+			return 0;
+#endif /* !IL_CONFIG_FP_SUPPORTED */
+
  		case IL_META_ELEMTYPE_STRING:
 			return (ILObject *)_ILStringInternFromConstant(thread,ptr,len / 2);
 
@@ -2630,7 +2661,11 @@ ILObject *_IL_MethodBase_GetMethodFromHandle(ILExecThread *thread,
 									         void *handle)
 {
 	ILMethod *method;
+#ifdef IL_USE_JIT
+	method = ILProgramItemToMethod((ILProgramItem *)handle);
+#else
 	method = ILProgramItemToMethod((ILProgramItem *)(*((void **)handle)));
+#endif
 	if(method)
 	{
 		if(ILMethod_IsConstructor(method) ||
@@ -2817,7 +2852,7 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 	numParams = (ILInt32)ILTypeNumParams(signature);
 	if(numParams == 0)
 	{
-		if(parameters && parameters->length != 0)
+		if(parameters && ArrayLength(parameters) != 0)
 		{
 			ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
 			return 0;
@@ -2825,7 +2860,7 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 	}
 	else
 	{
-		if(!parameters || parameters->length != numParams)
+		if(!parameters || ArrayLength(parameters) != numParams)
 		{
 			ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
 			return 0;
@@ -3022,6 +3057,71 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 		++argNum;
 	}
 
+#ifdef IL_USE_JIT
+	/* Invoke the method */
+	if(isCtor)
+	{
+		return ILExecThreadCallCtorV(thread, method, args);
+	}
+	else 
+	{
+		/* Handle the return object creation. */
+		ILMemZero(&result, sizeof(result));
+		paramType = ILTypeGetEnumType(ILTypeGetReturn(signature));
+		if(ILType_IsPrimitive(paramType))
+		{
+			/* Box a primitive value */
+			if(paramType == ILType_Void)
+			{
+				paramObject = 0;
+				result.ptrValue = (void *)&paramObject;
+			}
+			else
+			{
+				ILClass *classInfo = ILClassFromType
+					(ILContextNextImage(thread->process->context, 0),
+			 		0, paramType, 0);
+				if(!classInfo)
+				{
+					ILExecThreadThrowOutOfMemory(thread);
+					return 0;
+				}
+				classInfo = ILClassResolve(classInfo);
+				paramObject = (ILObject *)_ILEngineAllocObject
+										(thread, classInfo);
+				if(!paramObject)
+				{
+					return 0;
+				}
+				result.ptrValue = paramObject;
+			}
+		}
+		else if(ILType_IsValueType(paramType))
+		{
+			paramObject = (ILObject *)_ILEngineAllocObject
+				(thread, ILType_ToValueType(paramType));
+			if(!paramObject)
+			{
+				return 0;
+			}
+			result.ptrValue = paramObject;
+		}
+		else
+		{
+			paramObject = 0;
+			result.ptrValue = (void *)&paramObject;
+		}
+
+		/* and call the method now. */
+		if(ILExecThreadCallV(thread, method, result.ptrValue, args))
+		{
+			/* An exception was thrown by the method */
+			return 0;
+		}
+	}
+
+	return paramObject;
+#else
 	/* Allocate a boxing object for the result if it is a value type */
 	ILMemZero(&result, sizeof(result));
 	paramType = ILTypeGetEnumType(ILTypeGetReturn(signature));
@@ -3084,6 +3184,7 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 		/* Don't know how to box this type of value */
 		return 0;
 	}
+#endif
 }
 
 /*
