@@ -215,6 +215,12 @@ static void __PrivateThreadStart(void *objectArg)
 
 	thread = ILExecThreadCurrent();
 
+#ifdef IL_USE_JIT
+	/* Set the exception handler which converts builtin
+	   libjit exceptions into clr exceptions */
+	jit_exception_set_handler(_ILJitExceptionHandler);
+#endif
+
 	_ILCallMethod
 		(
 			thread,
@@ -955,7 +961,7 @@ ILBool _IL_WaitHandle_InternalWaitAll(ILExecThread *_thread,
 	handles = (ILWaitHandle **)ArrayToBuffer(waitHandles);
 
 	/* Perform the wait */
-	result = ILWaitAll(handles, (ILUInt32)(waitHandles->length), timeout);
+	result = ILWaitAll(handles, (ILUInt32)(ArrayLength(waitHandles)), timeout);
 
 	if (result == IL_WAIT_TIMEOUT)
 	{
@@ -994,7 +1000,7 @@ ILInt32 _IL_WaitHandle_InternalWaitAny(ILExecThread *_thread,
 	handles = (ILWaitHandle **)ArrayToBuffer(waitHandles);
 
 	/* Perform the wait */
-	result = ILWaitAny(handles, (ILUInt32)(waitHandles->length), timeout);
+	result = ILWaitAny(handles, (ILUInt32)(ArrayLength(waitHandles)), timeout);
 
 	if (result == IL_WAIT_TIMEOUT)
 	{
