@@ -163,6 +163,12 @@ void _ILThreadSetExecContext(ILThread *thread, ILThreadExecContext *context, ILT
 		}
 	#endif
 
+#ifdef IL_USE_JIT
+	/* Set the exception handler which converts builtin
+	   libjit exceptions into clr exceptions */
+	jit_exception_set_handler(_ILJitExceptionHandler);
+#endif
+
 	context->execThread->supportThread = thread;
 }
 
@@ -654,6 +660,12 @@ ILExecThread *_ILExecThreadCreate(ILExecProcess *process, int ignoreProcessState
 	thread->abortHandlerEndPC = 0;
 	thread->threadAbortException = 0;
 	thread->abortHandlerFrame = 0;
+
+#ifdef IL_CONFIG_DEBUGGER
+	thread->numWatches = 0;
+	thread->maxWatches = 0;
+#endif
+
 	/* Attach the thread to the process */
 	ILMutexLock(process->lock);
 	if (!ignoreProcessState
@@ -712,6 +724,14 @@ void _ILExecThreadDestroy(ILExecThread *thread)
 	/* Destroy the call frame stack */
 	ILGCFreePersistent(thread->frameStack);
 
+#ifdef IL_CONFIG_DEBUGGER
+	/* Destroy the watch stack */
+	if(thread->watchStack)
+	{
+		ILGCFreePersistent(thread->watchStack);
+	}
+#endif
+
 	/* Destroy the thread block */
 	ILGCFreePersistent(thread);
 
@@ -762,6 +782,49 @@ ILCallFrame *_ILGetNextCallFrame(ILExecThread *thread, ILCallFrame *frame)
 	}
 }
 
+#ifdef IL_CONFIG_DEBUGGER
+
+ILWatch *_ILAllocWatch(ILExecThread *thread)
+{
+	ILUInt32 newsize;
+	ILWatch *watches;
+
+	/* Calculate watch stack size */
+	if(thread->maxWatches)
+	{
+		newsize = thread->maxWatches * 2;
+	}
+	else
+	{
+		newsize = 32;		/* initial value */
+	}
+
+	/* Allocate new watches */
+	if((watches = (ILWatch *)ILGCAllocPersistent
+				(sizeof(ILWatch) * newsize)) == 0)
+	{
+	    thread->thrownException = thread->process->outOfMemoryObject;
+		return 0;
+	}
+
+	/* Copy the old contents (if any) */
+	if(thread->maxWatches)
+	{
+		ILMemCpy(watches, thread->watchStack,
+				sizeof(ILWatch) * thread->maxWatches);
+
+		/* Free the old watches */
+		ILGCFreePersistent(thread->watchStack);
+	}
+	thread->watchStack = watches;
+	thread->maxWatches = newsize;
+
+	/* Return the new watch to the caller */
+	return &(thread->watchStack[(thread->numWatches)]);
+}
+
+#endif	// IL_CONFIG_DEBUGGER
+
 ILMethod *ILExecThreadStackMethod(ILExecThread *thread, unsigned long num)
 {
 	ILCallFrame *frame;
