@@ -23,6 +23,10 @@
 #include "il_opcodes.h"
 #include "il_align.h"
 #include "il_debug.h"
+#include "debugger.h"
+#ifdef IL_USE_JIT
+#include "jitc.h"
+#endif
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -691,11 +695,12 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
 	coderFlags = ILCoderGetFlags(coder);
 	isStatic = ILMethod_IsStatic(method);
 	isSynchronized = ILMethod_IsSynchronized(method);
-		
+
 restart:
 	result = 0;
 	labelList = 0;
 	hasRethrow = 0;
+	isReadOnly = 0;
 
 	/* Initialize the memory allocator that is used for temporary
 	   allocation during bytecode verification */
@@ -1065,8 +1070,13 @@ restart:
 			VALIDATE_TARGET_STACK(offset);
 
 			/* Notify the coder of a label at this position */
+		#ifdef IL_USE_JIT
+			ILCoderStackRefresh(coder, stack, stackSize);
+			ILCoderLabel(coder, offset);
+		#else
 			ILCoderLabel(coder, offset);
 			ILCoderStackRefresh(coder, stack, stackSize);
+		#endif
 		}
 		else if(lastWasJump)
 		{
