@@ -18,7 +18,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "ildasm_internal.h"
+#include "il_dumpasm.h"
+#include "il_system.h"
+#include "il_utils.h"
 
 #ifdef	__cplusplus
 extern	"C" {
