@@ -350,18 +350,18 @@ OPDEF("unaligned.",			0, 0, IL_OPCODE_ARGS_UINT8, 3)
 OPDEF("volatile.",			0, 0, IL_OPCODE_ARGS_NONE, 2)
 OPDEF("tail.",				0, 0, IL_OPCODE_ARGS_NONE, 2)
 OPDEF("initobj",			1, 0, IL_OPCODE_ARGS_TOKEN, 6)
-OPDEF("ann.live",			0, 0, IL_OPCODE_ARGS_ANN_LIVE, 4)
+OPDEF("constrained.",		0, 0, IL_OPCODE_ARGS_TOKEN, 6)
 OPDEF("cpblk",				3, 0, IL_OPCODE_ARGS_NONE, 2)
 OPDEF("initblk",			3, 0, IL_OPCODE_ARGS_NONE, 2)
-OPDEF("ann.ref",			0, 0, IL_OPCODE_ARGS_ANN_REF, 4)
+OPDEF("no.",				0, 0, IL_OPCODE_ARGS_UINT8, 3)
 OPDEF("rethrow",			0, 0, IL_OPCODE_ARGS_NONE, 2)
 
 OPDEF("unused_prefix_1B",	0, 0, IL_OPCODE_ARGS_INVALID, 2)
 
 OPDEF("sizeof",				0, 1, IL_OPCODE_ARGS_TOKEN, 6)
 OPDEF("refanytype",			1, 1, IL_OPCODE_ARGS_NONE, 2)
+OPDEF("readonly.",			0, 0, IL_OPCODE_ARGS_NONE, 2)
 
-OPDEF("unused_prefix_1E",	0, 0, IL_OPCODE_ARGS_INVALID, 2)
 OPDEF("unused_prefix_1F",	0, 0, IL_OPCODE_ARGS_INVALID, 2)
 OPDEF("unused_prefix_20",	0, 0, IL_OPCODE_ARGS_INVALID, 2)
 OPDEF("unused_prefix_21",	0, 0, IL_OPCODE_ARGS_INVALID, 2)
