@@ -48,16 +48,17 @@ typedef struct _tagILCoderClass ILCoderClass;
  */
 typedef enum
 {
-	ILEngineType_I4,
-	ILEngineType_I8,
-	ILEngineType_I,
-	ILEngineType_F,
-	ILEngineType_M,
-	ILEngineType_O,
-	ILEngineType_T,
-	ILEngineType_MV,
-	ILEngineType_Invalid,
-	ILEngineType_TypedRef,
+	ILEngineType_I4,		/* Int32 value. */
+	ILEngineType_I8,		/* Int64 value. */
+	ILEngineType_I,			/* Native integer. */
+	ILEngineType_F,			/* Native float. */
+	ILEngineType_M,			/* Managed reference. */
+	ILEngineType_O,			/* Object reference. */
+	ILEngineType_T,			/* Typed reference. */
+	ILEngineType_MV,		/* structure or value type. */
+	ILEngineType_Invalid,	/* Invalid. Used for void return values. */
+	ILEngineType_TypedRef,	/* TypeRef structure. */
+	ILEngineType_CM			/* Controlled-mutability managed pointer. */
 
 } ILEngineType;
 #define	ILEngineType_ValidTypes	8
@@ -90,6 +91,7 @@ typedef struct
 	ILUInt32 numVarArgs;
 	int hasParamArray;
 	int tailCall;
+	ILType *signature;
 
 } ILCoderMethodInfo;
 
@@ -808,6 +810,32 @@ struct _tagILCoderClass
 						  ILUInt32 customNameLen, ILUInt32 customCookieLen,
 						  void *customName, void *customCookie);
 
+	/*
+	 * Run the cctors collected during code generation.
+	 * This function must be called after the method is compiled successfully
+	 * and before the method itself is executed.
+	 * The metadata lock must be still accuired and will be released just
+	 * before the cctors are executed.
+     */
+	ILInt32 (*runCCtors)(ILCoder *coder, void *userData);
+
+	/*
+	 * Run the class initializer for the given class.
+	 */
+	ILInt32 (*runCCtor)(ILCoder *coder, ILClass *classInfo);
+
+	/*
+	 * Handle a locked method.
+	 * If the current thread holds the method lock the userData supplied when
+	 * the method was locked is returned.
+	 * If the current thread is currently executing class initializers the
+	 * cctors of the method are executed and after they are finished the
+	 * the userData supplied when the method was locked is returned.
+	 * In the other case the thread is blocked until the method is unlocked
+	 * and then the userData supplied when the method was locked is returned.
+	 */
+	void *(*handleLockedMethod)(ILCoder *coder, ILMethod *method);
+
 	/*
 	 * Sentinel string to catch missing methods in class tables.
 	 */
@@ -1087,6 +1115,12 @@ struct _tagILCoderClass
 													(customCookie)))
 #define	ILCoderMarkEnd(coder) \
 			((*((coder)->classInfo->markEnd))((coder)))
+#define	ILCoderRunCCtors(coder,userData) \
+			((*((coder)->classInfo->runCCtors))((coder), (userData)))
+#define	ILCoderRunCCtor(coder,classInfo) \
+			((*((coder)->classInfo->runCCtor))((coder), (classInfo)))
+#define ILCoderHandleLockedMethod(coder,method) \
+			(*((coder)->classInfo->handleLockedMethod))((coder), (method))
 
 #ifdef	__cplusplus
 };
