@@ -233,6 +233,9 @@ void GC_push_thread_structures GC_PROTO((void))
 # endif
 }
 
+/* Defined in misc.c */
+extern CRITICAL_SECTION GC_write_cs;
+
 void GC_stop_world()
 {
   DWORD thread_id = GetCurrentThreadId();
@@ -241,6 +244,9 @@ void GC_stop_world()
   if (!GC_thr_initialized) ABORT("GC_stop_world() called before GC_thr_init()");
 
   GC_please_stop = TRUE;
+# ifndef CYGWIN32
+    EnterCriticalSection(&GC_write_cs);
+# endif /* !CYGWIN32 */
   for (i = 0; i <= GC_get_max_thread_index(); i++)
     if (thread_table[i].stack_base != 0
 	&& thread_table[i].id != thread_id) {
@@ -271,6 +277,9 @@ void GC_stop_world()
 #     endif
       thread_table[i].suspended = TRUE;
     }
+# ifndef CYGWIN32
+    LeaveCriticalSection(&GC_write_cs);
+# endif /* !CYGWIN32 */
 }
 
 void GC_start_world()
@@ -944,4 +953,35 @@ BOOL WINAPI DllMain(HINSTANCE inst, ULONG reason, LPVOID reserved)
 
 # endif /* !MSWINCE */
 
+/*
+ * Run a function under GC control in a thread that was not created
+ * through the gc routines.
+ * (for example by a thrid party library that does a callback).
+ */
+GC_API void *
+GC_run_thread(LPTHREAD_START_ROUTINE thread_func,  LPVOID arg)
+{
+    LPVOID result;
+
+    /* register the thread */
+#if defined(GC_DLL) && !defined(CYGWIN32)
+    /* we have nothing to do here because the thread is registered through DllMain */
+#else
+    GC_ASSERT(GC_thr_initialized);
+    if (GC_main_thread != GetCurrentThreadId()) {
+        GC_new_thread();
+    } /* o.w. we already did it during GC_thr_init(), called by GC_init() */
+#endif
+    result = thread_func(arg);
+
+    /* unregister the thread */
+#if defined(GC_DLL) && !defined(CYGWIN32)
+	/* we have nothing to do here because the thread is will be unregistered */
+    /* through DllMain */
+#else
+    GC_delete_thread(GetCurrentThreadId());
+#endif
+    return result;
+}
+
 #endif /* GC_WIN32_THREADS */
