@@ -156,8 +156,9 @@ void _ILThreadSuspendUntilResumed(ILThread *thread);
  * Primitive mutex operations.  Note: the "Lock" and "Unlock"
  * operations are not "suspend-safe".
  */
+extern pthread_mutexattr_t _ILMutexAttr;
 #define	_ILMutexCreate(mutex)	\
-			(pthread_mutex_init((mutex), (pthread_mutexattr_t *)0))
+			(pthread_mutex_init((mutex), &_ILMutexAttr))
 #define	_ILMutexDestroy(mutex)	\
 			(pthread_mutex_destroy((mutex)))
 #define	_ILMutexLockUnsafe(mutex)	\
@@ -205,6 +206,7 @@ void _ILThreadSuspendUntilResumed(ILThread *thread);
 #define	_ILSemaphoreDestroy(sem)	(sem_destroy((sem)))
 #define	_ILSemaphoreWait(sem)		(sem_wait((sem)))
 #define	_ILSemaphorePost(sem)		(sem_post((sem)))
+int _ILSemaphorePostMultiple(_ILSemaphore *sem, ILUInt32 count);
 
 /*
  * Primitive condition variable operations.
