@@ -33,6 +33,9 @@
 #define	HAVE_SETSOCKOPT		1
 #define	HAVE_GETSOCKOPT		1
 #else
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
 #if TIME_WITH_SYS_TIME
 	#include <sys/time.h>
     #include <time.h>
@@ -74,6 +77,17 @@
 #endif
 #endif
 
+#ifdef IL_WIN32_NATIVE
+	/* Win32 defines IPPROTO_TCP, not SOL_TCP */
+	#if !defined (SOL_TCP) && defined(IPPROTO_TCP)
+		#define SOL_TCP IPPROTO_TCP
+	#endif
+	/* Win32 defines IPPROTO_UDP, not SOL_UDP */
+	#if !defined (SOL_UDP) && defined(IPPROTO_UDP)
+		#define SOL_UDP IPPROTO_UDP
+	#endif 
+#endif
+
 #ifdef	__cplusplus
 extern	"C" {
 #endif
@@ -359,6 +373,8 @@ int ILSysIOAddressFamilySupported(ILInt32 af)
 
 ILSysIOHandle ILSysIOSocket(ILInt32 domain, ILInt32 type, ILInt32 protocol)
 {
+	int iSocket = 0;
+
 #ifdef IL_WIN32_NATIVE
 	_ILWinSockInit();
 #endif
@@ -411,7 +427,11 @@ ILSysIOHandle ILSysIOSocket(ILInt32 domain, ILInt32 type, ILInt32 protocol)
 		break;
 	}
 	
-	return (ILSysIOHandle)(ILNativeInt)(socket(domain, type, protocol));
+	iSocket = socket(domain, type, protocol);
+#ifdef HAVE_FCNTL
+	if( iSocket >= 0 ) fcntl( iSocket, F_SETFD, FD_CLOEXEC );
+#endif
+	return (ILSysIOHandle)(ILNativeInt)(iSocket);
 }
 
 int ILSysIOSocketBind(ILSysIOHandle sockfd, unsigned char *addr,
